package java.util.jar;

import java.io.File;
import java.io.InputStream;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.OutputStream;
import java.util.SortedMap;
import com.sun.java.util.jar.pack.PackerImpl;
import com.sun.java.util.jar.pack.UnpackerImpl;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;

public abstract class Pack200
{
    private static final String PACK_PROVIDER = "java.util.jar.Pack200.Packer";
    private static final String UNPACK_PROVIDER = "java.util.jar.Pack200.Unpacker";
    private static Class<?> packerImpl;
    private static Class<?> unpackerImpl;
    
    private Pack200() {
    }
    
    public static synchronized Pack200.Packer newPacker() {
        return (Pack200.Packer)newInstance("java.util.jar.Pack200.Packer");
    }
    
    public static Pack200.Unpacker newUnpacker() {
        return (Pack200.Unpacker)newInstance("java.util.jar.Pack200.Unpacker");
    }
    
    private static synchronized Object newInstance(String s) {
        String s2 = "(unknown)";
        try {
            Class<Object> forName = "java.util.jar.Pack200.Packer".equals(s) ? Pack200.packerImpl : Pack200.unpackerImpl;
            if (forName == null) {
                s2 = (String)AccessController.doPrivileged(new GetPropertyAction(s, ""));
                if (s2 != null && !s2.equals("")) {
                    forName = Class.forName(s2);
                }
                else if ("java.util.jar.Pack200.Packer".equals(s)) {
                    forName = PackerImpl.class;
                }
                else {
                    forName = UnpackerImpl.class;
                }
            }
            return forName.newInstance();
        }
        catch (ClassNotFoundException cause) {
            throw new Error("Class not found: " + s2 + ":\ncheck property " + s + " in your properties file.", cause);
        }
        catch (InstantiationException cause2) {
            throw new Error("Could not instantiate: " + s2 + ":\ncheck property " + s + " in your properties file.", cause2);
        }
        catch (IllegalAccessException cause3) {
            throw new Error("Cannot access class: " + s2 + ":\ncheck property " + s + " in your properties file.", cause3);
        }
    }
    
    public interface Packer
    {
        public static final String SEGMENT_LIMIT = "pack.segment.limit";
        public static final String KEEP_FILE_ORDER = "pack.keep.file.order";
        public static final String EFFORT = "pack.effort";
        public static final String DEFLATE_HINT = "pack.deflate.hint";
        public static final String MODIFICATION_TIME = "pack.modification.time";
        public static final String PASS_FILE_PFX = "pack.pass.file.";
        public static final String UNKNOWN_ATTRIBUTE = "pack.unknown.attribute";
        public static final String CLASS_ATTRIBUTE_PFX = "pack.class.attribute.";
        public static final String FIELD_ATTRIBUTE_PFX = "pack.field.attribute.";
        public static final String METHOD_ATTRIBUTE_PFX = "pack.method.attribute.";
        public static final String CODE_ATTRIBUTE_PFX = "pack.code.attribute.";
        public static final String PROGRESS = "pack.progress";
        public static final String KEEP = "keep";
        public static final String PASS = "pass";
        public static final String STRIP = "strip";
        public static final String ERROR = "error";
        public static final String TRUE = "true";
        public static final String FALSE = "false";
        public static final String LATEST = "latest";
        
        public abstract SortedMap<String, String> properties();
        
        public abstract void pack(JarFile p0, OutputStream p1) throws IOException;
        
        public abstract void pack(JarInputStream p0, OutputStream p1) throws IOException;
        
        @Deprecated
        default public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        }
        
        @Deprecated
        default public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        }
    }
    
    public interface Unpacker
    {
        public static final String KEEP = "keep";
        public static final String TRUE = "true";
        public static final String FALSE = "false";
        public static final String DEFLATE_HINT = "unpack.deflate.hint";
        public static final String PROGRESS = "unpack.progress";
        
        public abstract SortedMap<String, String> properties();
        
        public abstract void unpack(InputStream p0, JarOutputStream p1) throws IOException;
        
        public abstract void unpack(File p0, JarOutputStream p1) throws IOException;
        
        @Deprecated
        default public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        }
        
        @Deprecated
        default public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        }
    }
}
